require 'uu/os/qos/qos_handler'
require 'uu/os/uesuri'
require 'uu/os/rest/remote_client'
require 'uu/uds/bucket/bucket_create'
require 'uu/uds/bucket/bucket_attributes'
require 'uu/uds/bucket/bucket_set_attributes'
require 'uu/uds/bucket/blob_create'
require 'uu/uds/bucket/blob_attributes'

# UU module
module UU
  # UDS module
  module UDS

    # Bucket is used for storing (and sharing) binary data of application.
    module Bucket
      extend self

      # Service path
      PATH = '/uu/uds/Bucket'

      # Creates new bucket for given application. Bucket code and bucket name must be given in order to
      # create new bucket.
      #
      # @param [String, UU::OS::UESURI] application_uri URI of application for which to create bucket.
      # @param [BucketCreate] bucket DTO containing attributes of the new bucket.
      #
      # @return [UU::OS::UESURI] URI of created bucket.
      def create(application_uri, bucket = nil)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        payload = Bucket::BucketCreate.new(bucket).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post(:create, application_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes of bucket. In case bucket with given URI does not exist, null is returned.
      #
      # @param [String, UU::OS::UESURI] bucket_uri URI of required bucket.
      #
      # @return [BucketAttributes] DTO with attributes of bucket.
      def get_attributes(bucket_uri)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', bucket_uri)
          return Bucket::BucketAttributes.new(res)
        end
      end

      # Updates bucket attributes and configuration. In case bucket with given URI does not exist,
      # exception is thrown.
      #
      # @param [String, UU::OS::UESURI] bucket_uri URI of bucket to be updated
      # @param [BucketSetAttributes] bucket DTO containing new bucket configuration
      #
      # @return [UU::OS::UESURI] URI of the updated bucket
      def set_attributes(bucket_uri, bucket = nil)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        payload = Bucket::BucketSetAttributes.new(bucket).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post(:setAttributes, bucket_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes existing bucket. Bucket must be empty to be able to delete it.
      #
      # @param [String, UU::OS::UESURI] bucket_uri URI of bucket to be deleted.
      def delete(bucket_uri)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.delete('delete', bucket_uri)
        end
      end

      # Creates new blob (object representing binary data). Only application which
      # is set as bucket owner is allowed to create new blob.
      #
      # @param [String, UU::OS::UESURI] bucket_uri URI of bucket where to insert blob.
      # @param [BlobCreate] blob DTO containing attributes of the new blob.
      #
      # @return [UU::OS::UESURI] URI of created blob.
      def create_blob(bucket_uri, blob = nil)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        svc.timeout = -1 # Upload may take long time
        bv = Bucket::BlobCreate.new(blob)
        bv.serialize_as_stream_handler = false
        payload = bv.to_hash(false)
        res = svc.post(:createBlob, bucket_uri, payload)
        return UU::OS::UESURI.new(res)
      end

      # Returns attributes of blob. In case blob with given URI does not exist, null is returned.
      # Returned DTO does not contain actual binary content of blob. Use method get_blob_data to
      # download blob binary content. Only application which is set as bucket owner is allowed
      # to read blob attributes.
      #
      # @param [String, UU::OS::UESURI] blob_uri URI of required blob.
      #
      # @return [BlobAttributes] DTO with attributes of blob.
      def get_blob_attributes(blob_uri)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get(:getBlobAttributes, blob_uri)
          return Bucket::BlobAttributes.new(res)
        end
      end

      # Returns binary content of blob (along with binary metadata). In case blob with given URI
      # does not exist, null is returned.  Only application which is set as bucket owner is allowed
      # to get blob binary content.
      #
      # @param [String, UU::OS::UESURI] blob_uri URI of required blob.
      #
      # @return [UU::OS::REST::BinaryValue] Blob binary content.
      def get_blob_data(blob_uri)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        svc.timeout = -1 # Download may take long time
        svc.add_header(:accept, :"application/octet-stream")
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary(:getBlobData, blob_uri)
        end
      end

      # Deletes existing blob. Only application which is set as bucket owner is allowed to delete blob.
      #
      # @param [String, UU::OS::UESURI] blob_uri URI of blob to be deleted.
      def delete_blob(blob_uri)
        svc = UU::OS::REST::RemoteClient.new(Bucket)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.delete(:deleteBlob, blob_uri)
        end
      end

    end

  end
end
